(*
   module Lolevel
   Low-level facilities used by other modules.
*)

(* --------------------------------------------------------------- *)

module type PRINTING = sig
  val print : string -> unit
  val print_int : int -> unit

  val obox0 : unit -> unit
  val obox : unit -> unit
  val cbox : unit -> unit
  val break : unit -> unit
  val space : unit -> unit
  val flush : unit -> unit
  val newline : unit -> unit
  val force_newline : unit -> unit
end

module Printing : PRINTING

(* --------------------------------------------------------------- *)
(* Error printing utilities -- opened everywhere by convention *)

module Error : sig
  (* An exception raised by the low-level error printer; exported
     here so that it can be caught in module Main and converted into
     an exit status for the whole program. *)
  exception Exit of int

  (* An element of the type info represents a "file position": a
     file name, line number, and character position within the line.
     Used for printing error messages. *)
  type info = Lexing.position
  val void_fi : info

  (* A convenient datatype for a "value with file info." Used in
     the lexer and parser. *)
  type 'a withinfo = {i: info; v: 'a}

  type printer = unit -> unit

  (* Execute printer and fail. The printing function is
     called in a context where the formatter is processing an hvbox.
     Insert calls to Format.print_space to print a space or, if
     necessary, break the line at that point. *)
  val die : printer -> 'a
  val print_info : info -> unit

  (* Special case of die, simply print an error message and fail. *)
  val error : string -> 'a
  val error_with : info -> string -> 'a

  (* Variants that print a message but do not fail afterwards. *)
  val warning : string -> unit
  val warning_with : info -> string -> unit
end
